from unittest import TestCase
from contentful.space import Space


class SpaceTest(TestCase):
    def test_space(self):
        space = Space({
            "sys": {
                "type": "Space",
                "id": "cfexampleapi"
            },
            "name": "Contentful Example API",
            "locales": [
                {
                    "code": "en-US",
                    "default": True,
                    "name": "English",
                    "fallbackCode": None
                },
                {
                    "code": "tlh",
                    "default": False,
                    "name": "Klingon",
                    "fallbackCode": "en-US"
                }
            ]
        })

        self.assertEqual(space.id, 'cfexampleapi')
        self.assertEqual(space.name, 'Contentful Example API')
        self.assertEqual(len(space.locales), 2)
        self.assertEqual(str(space.locales[0]), "<Locale[English] code='en-US' default=True fallback_code=None optional=False>")
        self.assertEqual(str(space.locales[1]), "<Locale[Klingon] code='tlh' default=False fallback_code='en-US' optional=False>")
        self.assertEqual(str(space), "<Space[Contentful Example API] id='cfexampleapi'>")



import json
from unittest import TestCase
from contentful.errors import (
    HTTPError,
    NotFoundError,
    BadRequestError,
    AccessDeniedError,
    UnauthorizedError,
    RateLimitExceededError,
    ServerError,
    ServiceUnavailableError,
    get_error
)
from contentful.utils import retry_request, json_error_class
from contentful.client import Client


class MockResponse(object):
    def __init__(self, status_code, json, headers=None, invalid_json=False):
        self.status_code = status_code
        self._json = json
        self._invalid_json = invalid_json
        self.headers = headers if headers is not None else {}

    def json(self):
        if self._invalid_json:
            raise json_error_class()('foo', 'foo', 0)
        return json.loads(json.dumps(self._json))

    @property
    def text(self):
        return self._json


http_attempts = 0


def mock_http_call(url, query):
    global http_attempts
    if http_attempts < query.get('fail_until', 1):
        http_attempts += 1
        raise RateLimitExceededError(
            MockResponse(429, {'message': 'foo'}, headers={'x-contentful-ratelimit-reset': query.get('reset', 0.1)})
        )
    return 'succeed'


class ErrorsTest(TestCase):
    def test_default_additional_info_is_empty(self):
        response = MockResponse(512, "not json", invalid_json=True)
        error = get_error(response)

        self.assertEqual(error._additional_error_info(), [])

    def test_default_error_message(self):
        response = MockResponse(512, "not json", invalid_json=True)
        error = get_error(response)

        expected_error = "\n".join([
            "HTTP status code: 512",
            "Message: The following error was received: not json"
        ])
        self.assertEqual(str(error), expected_error)

    def test_generic_details(self):
        response = MockResponse(512, {
            'details': 'some text'
        })
        error = get_error(response)

        expected_error = "\n".join([
            "HTTP status code: 512",
            "Message: The following error was received: {'details': 'some text'}",
            "Details: some text"
        ])
        self.assertEqual(str(error), expected_error)

    def test_not_found_error(self):
        response = MockResponse(404, {
            'message': 'The resource could not be found.',
            'details': {
              'type': 'Asset',
              'id': 'foobar'
            },
            'requestId': '$foobar123'
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 404)
        expected_error = "\n".join([
            "HTTP status code: 404",
            "Message: The resource could not be found.",
            "Details: The requested Asset could not be found. ID: foobar.",
            "Request ID: $foobar123"
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, NotFoundError))

    def test_not_found_error_with_sys_on_details(self):
        response = MockResponse(404, {
            'message': 'The resource could not be found.',
            'details': {
              'sys': {
                'type': 'Space',
                'id': 'foobar'
              }
            },
            'requestId': '$foobar123'
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 404)
        expected_error = "\n".join([
            "HTTP status code: 404",
            "Message: The resource could not be found.",
            "Details: The requested Space could not be found. ID: foobar.",
            "Request ID: $foobar123"
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, NotFoundError))

    def test_not_found_error_details_is_a_string(self):
        response = MockResponse(404, {
            'message': 'The resource could not be found.',
            'details': 'The resource could not be found',
            'requestId': '$foobar123'
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 404)
        expected_error = "\n".join([
            "HTTP status code: 404",
            "Message: The resource could not be found.",
            "Details: The resource could not be found",
            "Request ID: $foobar123"
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, NotFoundError))

    def test_bad_request_error(self):
        response = MockResponse(400, {
            'message': 'The query you sent was invalid. Probably a filter or ordering specification is not applicable to the type of a field.',
            'details': {
                'errors': [
                    {
                        'details': 'The path "invalid_param" is not recognized'
                    }
                ]
            },
            'requestId': '$foobar234'
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 400)
        expected_error = "\n".join([
            "HTTP status code: 400",
            "Message: The query you sent was invalid. Probably a filter or ordering specification is not applicable to the type of a field.",
            'Details: The path "invalid_param" is not recognized',
            "Request ID: $foobar234"
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, BadRequestError))

    def test_bad_request_error_details_is_string(self):
        response = MockResponse(400, {
            'message': 'The query you sent was invalid. Probably a filter or ordering specification is not applicable to the type of a field.',
            'details': "some error",
            'requestId': '$foobar234'
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 400)
        expected_error = "\n".join([
            "HTTP status code: 400",
            "Message: The query you sent was invalid. Probably a filter or ordering specification is not applicable to the type of a field.",
            'Details: some error',
            "Request ID: $foobar234"
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, BadRequestError))

    def test_bad_request_error_errors_details_is_string(self):
        response = MockResponse(400, {
            'message': 'The query you sent was invalid. Probably a filter or ordering specification is not applicable to the type of a field.',
            'details': {
                'errors': [
                    'some error'
                ]
            },
            'requestId': '$foobar234'
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 400)
        expected_error = "\n".join([
            "HTTP status code: 400",
            "Message: The query you sent was invalid. Probably a filter or ordering specification is not applicable to the type of a field.",
            'Details: some error',
            "Request ID: $foobar234"
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, BadRequestError))

    def test_access_denied_error(self):
        response = MockResponse(403, {
            'message': 'Access Denied',
            'details': {
                'reasons': [
                    'foo',
                    'bar'
                ]
            }
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 403)
        expected_error = "\n".join([
            "HTTP status code: 403",
            "Message: Access Denied",
            "Details: ",
            "\tReasons:",
            "\t\tfoo",
            "\t\tbar"
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, AccessDeniedError))

    def test_unauthorized_error(self):
        response = MockResponse(401, {
            'message': 'The access token you sent could not be found or is invalid.',
            'requestId': '$foobar123'
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 401)
        expected_error = "\n".join([
            'HTTP status code: 401',
            'Message: The access token you sent could not be found or is invalid.',
            'Request ID: $foobar123'
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, UnauthorizedError))

    def test_rate_limit_exceeded_error(self):
        response = MockResponse(429, {
            'message': 'Rate Limit Exceeded'
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 429)
        expected_error = "\n".join([
            "HTTP status code: 429",
            "Message: Rate Limit Exceeded"
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, RateLimitExceededError))

    def test_rate_limit_exceeded_error_with_time(self):
        response = MockResponse(429, {}, headers={
            'x-contentful-ratelimit-reset': 60
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 429)
        expected_error = "\n".join([
            "HTTP status code: 429",
            "Message: Rate limit exceeded. Too many requests.",
            "Time until reset (seconds): 60"
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, RateLimitExceededError))

    def test_server_error(self):
        response = MockResponse(500, {
            'message': 'Server Error'
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 500)
        expected_error = "\n".join([
            "HTTP status code: 500",
            "Message: Server Error"
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, ServerError))

    def test_service_unavailable_error(self):
        response = MockResponse(503, {
            'message': 'Service Unavailable'
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 503)
        expected_error = "\n".join([
            "HTTP status code: 503",
            "Message: Service Unavailable"
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, ServiceUnavailableError))

    def test_other_error(self):
        response = MockResponse(418, {
            'message': "I'm a Teapot"
        })

        error = get_error(response)

        self.assertEqual(error.status_code, 418)
        expected_error = "\n".join([
            "HTTP status code: 418",
            "Message: I'm a Teapot"
        ])
        self.assertEqual(str(error), expected_error)
        self.assertTrue(isinstance(error, HTTPError))

    def test_rate_limit_retries(self):
        global http_attempts
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)

        http_attempts = 0
        result = retry_request(client)(mock_http_call)('/foo', {})

        self.assertEqual(http_attempts, 1)
        self.assertEqual(result, 'succeed')

    def test_rate_limit_max_retries(self):
        global http_attempts
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)

        http_attempts = 0
        self.assertRaises(RateLimitExceededError, retry_request(client)(mock_http_call), '/foo', {'fail_until': 2})

    def test_rate_limit_max_wait(self):
        global http_attempts
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)

        http_attempts = 0
        self.assertRaises(RateLimitExceededError, retry_request(client)(mock_http_call), '/foo', {'reset': 100})

    def test_predefined_errors_default_message(self):
        messages = {
            400: "The request was malformed or missing a required parameter.",
            401: "The authorization token was invalid.",
            403: "The specified token does not have access to the requested resource.",
            404: "The requested resource or endpoint could not be found.",
            429: "Rate limit exceeded. Too many requests.",
            500: "Internal server error.",
            502: "The requested space is hibernated.",
            503: "The request was malformed or missing a required parameter."
        }

        for status_code, message in messages.items():
            response = MockResponse(status_code, "foo", invalid_json=True)

            error = get_error(response)

            expected_error = "\n".join([
                "HTTP status code: {0}".format(status_code),
                "Message: {0}".format(message)
            ])
            self.assertEqual(str(error), expected_error)



import sys
import os

from .asset_test import AssetTest
from .client_test import ClientTest
from .content_type_cache_test import ContentTypeCacheTest
from .content_type_field_types_test import BasicFieldTest, LinkFieldTest, RichTextFieldTest, SymbolFieldTest, ArrayFieldTest, ObjectFieldTest
from .content_type_field_test import ContentTypeFieldTest
from .content_type_test import ContentTypeTest
from .deleted_asset_test import DeletedAssetTest
from .deleted_entry_test import DeletedEntryTest
from .entry_test import EntryTest
from .errors_test import ErrorsTest
from .locale_test import LocaleTest
from .resource_builder_test import ResourceBuilderTest
from .resource_test import ResourceTest
from .space_test import SpaceTest
from .sync_page_test import SyncPageTest
from .utils_test import UtilsTest

sys.path.insert(0, os.path.abspath('..'))

__all__ = [
    'AssetTest',
    'ClientTest',
    'ContentTypeCacheTest',
    'BasicFieldTest',
    'LinkFieldTest',
    'RichTextFieldTest',
    'SymbolFieldTest',
    'ArrayFieldTest',
    'ObjectFieldTest',
    'ContentTypeFieldTest',
    'ContentTypeTest',
    'DeletedAssetTest',
    'DeletedEntryTest',
    'EntryTest',
    'ErrorsTest',
    'LocaleTest',
    'ResourceBuilderTest',
    'ResourceTest',
    'SpaceTest',
    'SyncPageTest',
    'UtilsTest'
]



from unittest import TestCase
from contentful.array import Array
from contentful.entry import Entry


class ArrayTest(TestCase):
    def test_asset(self):
        entry = Entry({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'foo'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1
            },
            'fields': {
                'name': 'foobar',
                'date': '2016-06-06'
            }
        })
        array = Array({
            'sys': {
                'type': 'Array'
            },
            'items': [
                entry.raw
            ],
            'total': 11,
            'skip': 10,
            'limit': 10
        }, [entry])

        self.assertEqual(len(array), 1)
        self.assertEqual(array.total, 11)
        self.assertEqual(array.skip, 10)
        self.assertEqual(array.limit, 10)
        self.assertEqual(array[0], entry)
        self.assertEqual(str(array), "<Array size='1' total='11' limit='10' skip='10'>")
        for e in array:
            # testing __iter__
            # should be only one, so assert is valid
            self.assertEqual(e, entry)



import datetime
from dateutil.tz import tzutc
from unittest import TestCase
from contentful.deleted_entry import DeletedEntry


class DeletedEntryTest(TestCase):
    def test_deleted_entry(self):
        deleted_entry = DeletedEntry({
            "sys": {
                "space": {
                    "sys": {
                        "type": "Link",
                        "linkType": "Space",
                        "id": "cfexampleapi"
                        }
                    },
                "id": "5c6VY0gWg0gwaIeYkUUiqG",
                "type": "DeletedEntry",
                "createdAt": "2013-09-09T16:17:12.600Z",
                "updatedAt": "2013-09-09T16:17:12.600Z",
                "deletedAt": "2013-09-09T16:17:12.600Z",
                "revision": 1
            }
        })

        self.assertEqual(deleted_entry.id, '5c6VY0gWg0gwaIeYkUUiqG')
        self.assertEqual(deleted_entry.deleted_at, datetime.datetime(2013, 9, 9, 16, 17, 12, 600000, tzinfo=tzutc()))
        self.assertEqual(str(deleted_entry), "<DeletedEntry id='5c6VY0gWg0gwaIeYkUUiqG'>")



import vcr
from unittest import TestCase
from contentful.content_type_cache import ContentTypeCache
from contentful import Client


class ContentTypeCacheTest(TestCase):
    @vcr.use_cassette('fixtures/cache/cache.yaml')
    def test_cache(self):
        Client('o4h6g9w3pooi', 'b4c0n73n7fu1')

        cat_ct = ContentTypeCache.get('o4h6g9w3pooi', 'article')
        self.assertEqual(str(cat_ct), "<ContentType[Article] id='article'>")

    @vcr.use_cassette('fixtures/cache/cache.yaml')
    def test_cache_update(self):
        ContentTypeCache.__CACHE__ = {}
        client = Client('o4h6g9w3pooi', 'b4c0n73n7fu1', content_type_cache=False)

        ContentTypeCache.update_cache(client)

        cat_ct = ContentTypeCache.get('o4h6g9w3pooi', 'article')
        self.assertEqual(str(cat_ct), "<ContentType[Article] id='article'>")



from unittest import TestCase
from contentful.asset import Asset


class AssetTest(TestCase):
    def test_asset(self):
        asset = Asset({
            'sys': {
                'type': 'Asset',
                'id': 'foo',
                'createdAt': '2016-12-01T11:39:20.257859',
                'updatedAt': '2016-12-01T11:39:20.257859',
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                }
            },
            'fields': {
                'title': 'Awesome Pic',
                'description': 'A picture of something',
                'file': {
                    'fileName': 'foobar',
                    'contentType': 'image/jpeg',
                    'url': '//images.contentful.com/...',
                    'details': {
                        'size': 300
                    }
                }
            }
        })

        self.assertEqual(asset.id, 'foo')
        self.assertEqual(asset.title, 'Awesome Pic')
        self.assertEqual(asset.description, 'A picture of something')
        self.assertEqual(asset.url(), '//images.contentful.com/...')
        self.assertEqual(asset.url(w=123), '//images.contentful.com/...?w=123')
        self.assertEqual(str(asset), "<Asset id='foo' url='//images.contentful.com/...'>")

    def test_metadata_tags(self):
        asset = Asset({
            'sys': {
                'type': 'Asset',
                'id': 'foo',
                'createdAt': '2016-12-01T11:39:20.257859',
                'updatedAt': '2016-12-01T11:39:20.257859',
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                }
            },
            'metadata': {
                'tags': [
                    {
                        'sys': {
                            'type': 'Link',
                            'linkType': 'Tag',
                            'id': 'nyCampaign'
                        }
                    }
                ]
            },
            'fields': {
                'title': 'Awesome Pic',
                'metadata': 'A picture of something',
                'file': {
                    'fileName': 'foobar',
                    'contentType': 'image/jpeg',
                    'url': '//images.contentful.com/...',
                    'details': {
                        'size': 300
                    }
                }
            }
        })

        self.assertEqual(len(asset._metadata['tags']), 1)

        tag = asset._metadata['tags'][0]
        self.assertEqual(str(tag), "<Link[Tag] id='nyCampaign'>")
        self.assertEqual(tag.id, 'nyCampaign')
        self.assertEqual(tag.link_type, 'Tag')

    def test_asset_with_no_file_can_be_serialized_correctly(self):
        asset = Asset({
            'sys': {
                'type': 'Asset',
                'id': 'foo',
                'createdAt': '2016-12-01T11:39:20.257859',
                'updatedAt': '2016-12-01T11:39:20.257859',
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                }
            },
            'fields': {
                'title': 'Awesome Pic',
                'description': 'A picture of something'
            }
        })

        self.assertEqual(str(asset), "<Asset id='foo' url=''>")

    def test_asset_with_empty_file_can_be_serialized_correctly(self):
        asset = Asset({
            'sys': {
                'type': 'Asset',
                'id': 'foo',
                'createdAt': '2016-12-01T11:39:20.257859',
                'updatedAt': '2016-12-01T11:39:20.257859',
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                }
            },
            'fields': {
                'title': 'Awesome Pic',
                'description': 'A picture of something',
                'file': {}
            }
        })

        self.assertEqual(str(asset), "<Asset id='foo' url=''>")



from datetime import datetime
from unittest import TestCase
from contentful.entry import Entry
from contentful.content_type import ContentType
from contentful.content_type_cache import ContentTypeCache


class EntryTest(TestCase):
    def test_entry(self):
        ContentTypeCache.__CACHE__ = {}

        entry = Entry({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'foo'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1
            },
            'fields': {
                'name': 'foobar',
                'date': '2016-06-06'
            }
        })

        self.assertEqual(str(entry), "<Entry[foo] id='foobar'>")
        self.assertEqual(entry.name, 'foobar')
        self.assertEqual(entry.date, '2016-06-06')

    def test_metadata_tags(self):
        ContentTypeCache.__CACHE__ = {}

        entry = Entry({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'foo'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1
            },
            'metadata': {
                'tags': [
                    {
                        'sys': {
                            'type': 'Link',
                            'linkType': 'Tag',
                            'id': 'nyCampaign'
                        }
                    }
                ]
            },
            'fields': {
                'name': 'foobar',
                'date': '2016-06-06',
                'metadata': 'baz'
            }
        })

        self.assertEqual(len(entry._metadata['tags']), 1)

        tag = entry._metadata['tags'][0]
        self.assertEqual(str(tag), "<Link[Tag] id='nyCampaign'>")
        self.assertEqual(tag.id, 'nyCampaign')
        self.assertEqual(tag.link_type, 'Tag')

    def test_entry_unresolved_link(self):
        ContentTypeCache.__CACHE__ = {}

        errors = [
            {
                "sys": {
                    "id": "notResolvable",
                    "type": "error"
                },
                "details": {
                    "type": "Link",
                    "linkType": "Entry",
                    "id": "unresolvedLinkId"
                }
            }
        ]

        entry = Entry(
            {
                'sys': {
                    'space': {
                        'sys': {
                            'type': 'Link',
                            'linkType': 'Space',
                            'id': 'foo'
                        }
                    },
                    'contentType': {
                        'sys': {
                            'type': 'Link',
                            'linkType': 'ContentType',
                            'id': 'foo'
                        }
                    },
                    'type': 'Entry',
                    'createdAt': '2016-06-06',
                    'updatedAt': '2016-06-06',
                    'deletedAt': '2016-06-06',
                    'id': 'foobar',
                    'version': 1
                },
                'fields': {
                    'name': 'foobar',
                    'entryLink': {
                        'sys': {
                            'type': 'Link',
                            'linkType': 'Entry',
                            'id': 'unresolvedLinkId'
                        }
                    }
                }
            },
            None,
            errors
        )

        self.assertEqual(entry.fields().get('entryLink'), None)

    def test_cached_content_type_entry(self):
        ContentTypeCache.__CACHE__ = {}

        foo_ct = ContentType({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'type': 'ContentType',
                'id': 'foo'
            },
            "displayField": "name",
            "name": "Foo",
            "description": "",
            "fields": [
                {
                    "id": "name",
                    "name": "Name",
                    "type": "Text",
                    "localized": True,
                    "required": True,
                    "disabled": False,
                    "omitted": False
                },
                {
                    "id": "date",
                    "name": "Date",
                    "type": "Date",
                    "localized": True,
                    "required": True,
                    "disabled": False,
                    "omitted": False
                }
            ]
        })

        if 'foo' not in ContentTypeCache.__CACHE__:
            ContentTypeCache.__CACHE__['foo'] = []
        ContentTypeCache.__CACHE__['foo'].append(foo_ct)

        entry = Entry({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'foo'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1
            },
            'fields': {
                'name': 'foobar',
                'date': '2016-06-06'
            }
        })

        self.assertEqual(entry.name, 'foobar')
        self.assertEqual(entry.date, datetime(2016, 6, 6))

    def test_entry_includes(self):
        entry = Entry({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'foo'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1
            },
            'fields': {
                'name': 'foobar',
                'date': '2016-06-06',
                'entryLink': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Entry',
                        'id': 'linkedEntry'
                    }
                },
                'assetLink': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Asset',
                        'id': 'linkedAsset'
                    }
                },
                'arrayLink': [
                    {
                        'sys': {
                            'type': 'Link',
                            'linkType': 'Entry',
                            'id': 'otherLinkedEntry'
                        }
                    }
                ]
            }
        },
            includes=[
                {
                    'sys': {
                        'id': 'linkedEntry',
                        'type': 'Entry',
                        'contentType': {
                            'sys': {
                                'id': 'foobar'
                            }
                        },
                        'space': {
                            'sys': {
                                'type': 'Link',
                                'linkType': 'Space',
                                'id': 'foo'
                            }
                        },
                    },
                    'fields': {
                        'foo': 'bar'
                    }
                },
                {
                    'sys': {
                        'id': 'otherLinkedEntry',
                        'type': 'Entry',
                        'contentType': {
                            'sys': {
                                'id': 'foobar'
                            }
                        },
                        'space': {
                            'sys': {
                                'type': 'Link',
                                'linkType': 'Space',
                                'id': 'foo'
                            }
                        },
                    },
                    'fields': {
                        'foo': 'baz'
                    }
                },
                {
                    'sys': {
                        'id': 'linkedAsset',
                        'type': 'Asset'
                    },
                    'fields': {
                        'file': {
                            'url': '//images.contentful.com/...'
                        }
                    }
                }
            ])

        self.assertEqual(str(entry.entry_link), "<Entry[foobar] id='linkedEntry'>")
        self.assertEqual(str(entry.asset_link), "<Asset id='linkedAsset' url='//images.contentful.com/...'>")
        self.assertEqual(str(entry.array_link[0]), "<Entry[foobar] id='otherLinkedEntry'>")

    def test_entry_default_include_resolution_depth(self):
        default_entry = Entry({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'foo'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1
            },
            'fields': {
                'name': 'foobar',
                'date': '2016-06-06',
                'a': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Entry',
                        'id': 'linkedEntry'
                    }
                }
            }
        },
            includes=[
                {
                    'sys': {
                        'space': {
                            'sys': {
                                'id': 'foo'
                            }
                        },
                        'contentType': {
                            'sys': {
                                'id': 'foo'
                            }
                        },
                        'type': 'Entry',
                        'id': 'foobar',
                    },
                    'fields': {
                        'name': 'foobar',
                        'date': '2016-06-06',
                        'a': {
                            'sys': {
                                'type': 'Link',
                                'linkType': 'Entry',
                                'id': 'linkedEntry'
                            }
                        }
                    }
                },
                {
                    'sys': {
                        'id': 'linkedEntry',
                        'type': 'Entry',
                        'contentType': {
                            'sys': {
                                'id': 'foo'
                            }
                        }
                    },
                    'fields': {
                        'b': {
                            'sys': {
                                'type': 'Link',
                                'linkType': 'Entry',
                                'id': 'foobar'
                            }
                        }
                    }
                }
            ])

        self.assertEqual(str(default_entry.a), "<Entry[foo] id='linkedEntry'>")
        self.assertEqual(str(default_entry.a.b), "<Entry[foo] id='foobar'>")
        self.assertEqual(str(default_entry.a.b.a.b.a), "<Entry[foo] id='linkedEntry'>")
        self.assertEqual(str(default_entry.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b), "<Entry[foo] id='foobar'>")
        self.assertEqual(str(default_entry.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a), "<Link[Entry] id='linkedEntry'>")

    def test_defined_entry_include_resolution_depth(self):
        entry_only_5_nesting_levels = Entry({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'foo'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1
            },
            'fields': {
                'name': 'foobar',
                'date': '2016-06-06',
                'a': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Entry',
                        'id': 'linkedEntry'
                    }
                }
            }
        },
            includes=[
                {
                    'sys': {
                        'space': {
                            'sys': {
                                'id': 'foo'
                            }
                        },
                        'contentType': {
                            'sys': {
                                'id': 'foo'
                            }
                        },
                        'type': 'Entry',
                        'id': 'foobar',
                    },
                    'fields': {
                        'name': 'foobar',
                        'date': '2016-06-06',
                        'a': {
                            'sys': {
                                'type': 'Link',
                                'linkType': 'Entry',
                                'id': 'linkedEntry'
                            }
                        }
                    }
                },
                {
                    'sys': {
                        'id': 'linkedEntry',
                        'type': 'Entry',
                        'contentType': {
                            'sys': {
                                'id': 'foo'
                            }
                        }
                    },
                    'fields': {
                        'b': {
                            'sys': {
                                'type': 'Link',
                                'linkType': 'Entry',
                                'id': 'foobar'
                            }
                        }
                    }
                }
            ],
            max_depth=5)

        # self.assertEqual(str(entry_only_5_nesting_levels.a), "<Entry[foo] id='linkedEntry'>")
        # self.assertEqual(str(entry_only_5_nesting_levels.a.b), "<Entry[foo] id='foobar'>")
        # self.assertEqual(str(entry_only_5_nesting_levels.a.b.a.b.a), "<Entry[foo] id='linkedEntry'>")
        self.assertEqual(str(entry_only_5_nesting_levels.a.b.a.b.a.b), "<Link[Entry] id='foobar'>")

    def test_resource_link_without_resolution(self):
        entry = Entry({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'foo'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1
            },
            'fields': {
                'name': 'foobar',
                'date': '2016-06-06',
                'a': {
                    'sys': {
                        'type': 'ResourceLink',
                        'linkType': 'Contentful:Entry',
                        'urn': 'crn:contentful:::content:spaces/spaceId/environments/master/entries/entryId'
                    }
                }
            }
        })

        self.assertEqual(str(entry.a), "<ResourceLink[Contentful:Entry] urn='crn:contentful:::content:spaces/spaceId/environments/master/entries/entryId'>")

    def test_resource_link_with_resolution(self):
        entry = Entry({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'foo'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1
            },
            'fields': {
                'name': 'foobar',
                'date': '2016-06-06',
                'a': {
                    'sys': {
                        'type': 'ResourceLink',
                        'linkType': 'Contentful:Entry',
                        'urn': 'crn:contentful:::content:spaces/yb41ceqgyiw5/environments/master/entries/2JEG2a81ppm9XjGwU7NaMt'
                    }
                }
            }
        },
            includes=[
                {
                    'metadata': {
                        'tags': []
                    },
                    'sys': {
                        'space': {
                            'sys': {
                                'type': 'Link',
                                'linkType': 'Space',
                                'id': 'yb41ceqgyiw5'
                            }
                        },
                        'id': '2JEG2a81ppm9XjGwU7NaMt',
                        'type': 'Entry',
                        'createdAt': '2022-08-16T16:43:35.634Z',
                        'updatedAt': '2022-08-16T16:43:35.634Z',
                        'environment': {
                            'sys': {
                                'id': 'master',
                                'type': 'Link',
                                'linkType': 'Environment'
                            }
                        },
                        'revision': 1,
                        'contentType': {
                            'sys': {
                                'type': 'Link',
                                'linkType': 'ContentType',
                                'id': 'ct'
                            }
                        },
                        'locale': 'en-US'
                    },
                    'fields': {
                        'title': '1',
                        'slug': '1'
                    }
                }
            ])

        self.assertEqual(str(entry.a), "<Entry[ct] id='2JEG2a81ppm9XjGwU7NaMt'>")

    def test_resource_link_with_resolution_error(self):
        errors = [
            {
                'details': {
                    'urn': 'crn:contentful:::content:spaces/yb41ceqgyiw5/environments/master/entries/2JEG2a81ppm9XjGwU7NaMt',
                    'linkType': 'Contentful:Entry',
                    'type': 'ResourceLink'
                },
                'sys': {
                    'id': 'notResolvable',
                    'type': 'error'
                }
            }
        ]

        entry = Entry({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'foo'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1
            },
            'fields': {
                'name': 'foobar',
                'date': '2016-06-06',
                'a': {
                    'sys': {
                        'type': 'ResourceLink',
                        'linkType': 'Contentful:Entry',
                        'urn': 'crn:contentful:::content:spaces/yb41ceqgyiw5/environments/master/entries/2JEG2a81ppm9XjGwU7NaMt'
                    }
                }
            }
        },
            includes=None,
            errors=errors
        )

        self.assertEqual(entry.fields().get('a'), None)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from unittest import TestCase
from datetime import datetime
from contentful.content_type_field_types import (
    BasicField,
    SymbolField,
    TextField,
    IntegerField,
    NumberField,
    DateField,
    BooleanField,
    LocationField,
    LinkField,
    ArrayField,
    ObjectField,
    RichTextField,
    ResourceLinkField
)
from contentful.resource import Link


class BasicFieldTest(TestCase):
    def test_basic_field(self):
        self.assertEqual(BasicField().coerce('foo'), 'foo')
        self.assertEqual(BasicField().coerce(123), 123)

    def test_unicode_chars_dont_fail(self):
        self.assertEqual(TextField().coerce('😅'), '😅')


class SymbolFieldTest(TestCase):
    def test_symbol_field(self):
        self.assertEqual(SymbolField().coerce('foo'), 'foo')
        self.assertEqual(SymbolField().coerce(123), '123')

    def test_unicode_chars_dont_fail(self):
        self.assertEqual(TextField().coerce('😅'), '😅')


class TextFieldTest(TestCase):
    def test_text_field(self):
        self.assertEqual(TextField().coerce('foo'), 'foo')
        self.assertEqual(TextField().coerce(123), '123')

    def test_unicode_chars_dont_fail(self):
        self.assertEqual(TextField().coerce('😅'), '😅')


class IntegerFieldTest(TestCase):
    def test_integer_field(self):
        integer_field = IntegerField()
        self.assertEqual(integer_field.coerce(123), 123)
        self.assertEqual(integer_field.coerce(123.0), 123)
        self.assertEqual(integer_field.coerce('123'), 123)

        self.assertRaises(Exception, integer_field.coerce, 'foo')


class NumberFieldTest(TestCase):
    def test_number_field(self):
        number_field = NumberField()
        self.assertEqual(number_field.coerce(123), 123.0)
        self.assertEqual(number_field.coerce(123.0), 123.0)
        self.assertEqual(number_field.coerce('123'), 123.0)

        self.assertRaises(Exception, number_field.coerce, 'foo')


class DateFieldTest(TestCase):
    def test_date_field(self):
        date_field = DateField()
        self.assertEqual(date_field.coerce('2016-05-05'), datetime(2016, 5, 5))

        self.assertRaises(Exception, date_field.coerce, 'foo')


class BooleanFieldTest(TestCase):
    def test_boolean_field(self):
        self.assertEqual(BooleanField().coerce('foo'), True)
        self.assertEqual(BooleanField().coerce(True), True)
        self.assertEqual(BooleanField().coerce(''), False)
        self.assertEqual(BooleanField().coerce(False), False)


class LocationFieldTest(TestCase):
    def test_location_field(self):
        location_field = LocationField()
        self.assertEqual(location_field.coerce({'lat': 123, 'lon': 456}), (123, 456))

        location = location_field.coerce({'lat': 123, 'lon': 456})
        self.assertEqual(location.lat, 123)
        self.assertEqual(location.lon, 456)

        self.assertRaises(Exception, location_field.coerce, 'foo')


class LinkFieldTest(TestCase):
    def test_link_field(self):
        self.assertEqual(LinkField().coerce('foo'), 'foo')
        self.assertEqual(LinkField().coerce(123), 123)


class ArrayFieldTest(TestCase):
    def test_symbol_array_field(self):
        array_field = ArrayField({'type': 'Symbol'})

        self.assertEqual(array_field.coerce([123, 'foo']), ['123', 'foo'])

        self.assertRaises(Exception, array_field.coerce, 123)

    def test_link_array_field(self):
        array_field = ArrayField({'type': 'Link', 'linkType': 'Entry'})

        self.assertEqual(array_field.coerce([123, 'foo']), [123, 'foo'])

        self.assertRaises(Exception, array_field.coerce, 123)

    def test_link_array_field_other_type(self):
        self.assertRaises(Exception, ArrayField, {'type': 'foo'})


class ObjectFieldTest(TestCase):
    def test_object_field(self):
        object_field = ObjectField()

        self.assertEqual(object_field.coerce({'foo': 'bar'}), {'foo': 'bar'})
        self.assertEqual(object_field.coerce(123), 123)
        self.assertEqual(object_field.coerce('foo'), 'foo')
        self.assertEqual(object_field.coerce([1, 2, 3]), [1, 2, 3])
        self.assertEqual(object_field.coerce([{'foo': 'bar'}, {'baz': 'qux'}]), [{'foo': 'bar'}, {'baz': 'qux'}])


class RichTextFieldTest(TestCase):
    def test_rich_text_field(self):
        rt_field = RichTextField()

        self.assertEqual(repr(rt_field), '<RichTextField>')

        # on empty or non rich text field will return the value as is
        self.assertEqual(rt_field.coerce({}), {})
        self.assertEqual(rt_field.coerce(123), 123)

        # on proper rich text field, but without embedded entries will return as is
        document = {
            "nodeType": "document",
            "content": [{
                "nodeType": "paragraph",
                "nodeClass": "block",
                "content": [{
                    "nodeClass": "text",
                    "nodeType": "text",
                    "value": "This text is bold",
                    "marks": [{
                        "type": "bold"
                    }]
                }]
            }]
        }
        self.assertEqual(rt_field.coerce(document), document)

        # with embedded entries but with errors, will remove the node
        document = {
            "nodeType": "document",
            "content": [{
                "nodeType": "embedded-entry-block",
                "nodeClass": "block",
                "data": {
                    "target": {
                        "sys": {
                            "type": "Link",
                            "linkType": "Entry",
                            "id": "4JJ21pcEI0QSsea20g6K6K"
                        }
                    }
                }
            }]
        }

        errors = [{"details": {"id": "4JJ21pcEI0QSsea20g6K6K"}}]

        self.assertEqual(rt_field.coerce(document, errors=errors), {
            "nodeType": "document",
            "content": []
        })

        # With embedded entry that's unresolvable
        document = {
            "nodeType": "document",
            "content": [{
                "nodeType": "embedded-entry-block",
                "nodeClass": "block",
                "data": {
                    "target": {
                        "sys": {
                            "type": "Link",
                            "linkType": "Entry",
                            "id": "4JJ21pcEI0QSsea20g6K6K"
                        }
                    }
                }
            }]
        }

        coerced = rt_field.coerce(document)
        self.assertTrue(isinstance(coerced['content'][0]['data']['target'], Link))

        # with 2 embedded entries that have errors, both will be removed
        # without a list index out of range error
        document = {
            "nodeType": "document",
            "content": [{
                "nodeType": "embedded-entry-block",
                "nodeClass": "block",
                "data": {
                    "target": {
                        "sys": {
                            "type": "Link",
                            "linkType": "Entry",
                            "id": "4JJ21pcEI0QSsea20g6K6K"
                        }
                    }
                }
            }, {
                "nodeType": "embedded-entry-block",
                "nodeClass": "block",
                "data": {
                    "target": {
                        "sys": {
                            "type": "Link",
                            "linkType": "Entry",
                            "id": "4JJ21pcEI0QSsea20g6K6K"
                        }
                    }
                }
            }]
        }

        errors = [{"details": {"id": "4JJ21pcEI0QSsea20g6K6K"}}]

        self.assertEqual(rt_field.coerce(document, errors=errors), {
            "nodeType": "document",
            "content": []
        })


class ResourceLinkFieldTest(TestCase):
    def test_resource_link_field(self):
        resource_link_field = ResourceLinkField()

        self.assertEqual(resource_link_field.coerce('foo'), 'foo')



from unittest import TestCase
from contentful.sync_page import SyncPage


class SyncPageTest(TestCase):
    def test_sync_page_deleted_items(self):
        sync_page = SyncPage({
            "sys": {
                "type": "Array"
            },
            "items": [
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "4rPdazIwWkuuKEAQgemSmO",
                        "type": "DeletedEntry",
                        "createdAt": "2014-08-11T08:30:42.546Z",
                        "updatedAt": "2014-08-11T08:30:42.546Z",
                        "deletedAt": "2014-08-11T08:30:42.546Z",
                        "revision": 1
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "5c6VY0gWg0gwaIeYkUUiqG",
                        "type": "DeletedAsset",
                        "createdAt": "2013-09-09T16:17:12.600Z",
                        "updatedAt": "2013-09-09T16:17:12.600Z",
                        "deletedAt": "2013-09-09T16:17:12.600Z",
                        "revision": 1
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "finn",
                        "type": "DeletedAsset",
                        "createdAt": "2013-09-02T15:10:33.749Z",
                        "updatedAt": "2013-09-02T15:10:33.749Z",
                        "deletedAt": "2013-09-02T15:10:33.749Z",
                        "revision": 1
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "3MZPnjZTIskAIIkuuosCss",
                        "type": "DeletedAsset",
                        "createdAt": "2013-09-02T14:55:34.645Z",
                        "updatedAt": "2013-09-02T14:55:34.645Z",
                        "deletedAt": "2013-09-02T14:55:34.645Z",
                        "revision": 1
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "4gp6taAwW4CmSgumq2ekUm",
                        "type": "DeletedAsset",
                        "createdAt": "2013-09-02T14:55:34.623Z",
                        "updatedAt": "2013-09-02T14:55:34.623Z",
                        "deletedAt": "2013-09-02T14:55:34.623Z",
                        "revision": 2
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "1uf1qqyZuEuiwmigoUYkeu",
                        "type": "DeletedAsset",
                        "createdAt": "2013-09-02T14:55:34.323Z",
                        "updatedAt": "2013-09-02T14:55:34.323Z",
                        "deletedAt": "2013-09-02T14:55:34.323Z",
                        "revision": 1
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "4hlteQAXS8iS0YCMU6QMWg",
                        "type": "DeletedAsset",
                        "createdAt": "2013-09-02T14:55:34.282Z",
                        "updatedAt": "2013-09-02T14:55:34.282Z",
                        "deletedAt": "2013-09-02T14:55:34.282Z",
                        "revision": 1
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "CVebBDcQsSsu6yKKIayy",
                        "type": "DeletedEntry",
                        "createdAt": "2013-06-23T19:06:46.224Z",
                        "updatedAt": "2013-06-23T19:06:46.224Z",
                        "deletedAt": "2016-03-07T15:10:41.286Z",
                        "revision": 1
                    }
                }
            ],
            "nextPageUrl": (
                "https://cdn.contentful.com/spaces/cfexampleapi/environments/master/sync?"
                "sync_token=w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-"
                "wo7CnDjChMKKGsK1w5zCrA3CnU7CgEvDtsK6w7B2wrRZwrwPIgDCjVo8PMOoUcK2wqTCl8O1wpY8wpjCkGM"
            )
        })

        self.assertEqual(
            sync_page.next_page_url,
            "https://cdn.contentful.com/spaces/cfexampleapi/environments/master/sync?"
            "sync_token=w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-"
            "wo7CnDjChMKKGsK1w5zCrA3CnU7CgEvDtsK6w7B2wrRZwrwPIgDCjVo8PMOoUcK2wqTCl8O1wpY8wpjCkGM"
        )
        self.assertEqual(
            sync_page.next_sync_token,
            "w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-"
            "wo7CnDjChMKKGsK1w5zCrA3CnU7CgEvDtsK6w7B2wrRZwrwPIgDCjVo8PMOoUcK2wqTCl8O1wpY8wpjCkGM"
        )
        self.assertEqual(len(sync_page.items), 8)
        self.assertEqual(str(sync_page.items[-1]), "<DeletedEntry id='CVebBDcQsSsu6yKKIayy'>")
        self.assertEqual(str(sync_page.items[-2]), "<DeletedAsset id='4hlteQAXS8iS0YCMU6QMWg'>")

    def test_sync_page_initial(self):
        sync_page = SyncPage({
            "sys": {
                "type": "Array"
            },
            "items": [
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "5ETMRzkl9KM4omyMwKAOki",
                        "type": "Entry",
                        "createdAt": "2014-02-21T13:42:57.752Z",
                        "updatedAt": "2014-08-23T14:42:35.207Z",
                        "revision": 3,
                        "contentType": {
                            "sys": {
                                "type": "Link",
                                "linkType": "ContentType",
                                "id": "1t9IbcfdCk6m04uISSsaIK"
                            }
                        }
                    },
                    "fields": {
                        "name": {
                            "en-US": "London"
                        },
                        "center": {
                            "en-US": {
                                "lon": -0.12548719999995228,
                                "lat": 51.508515
                            }
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "7qVBlCjpWE86Oseo40gAEY",
                        "type": "Entry",
                        "createdAt": "2014-02-21T13:43:38.258Z",
                        "updatedAt": "2014-04-15T08:22:22.010Z",
                        "revision": 2,
                        "contentType": {
                            "sys": {
                                "type": "Link",
                                "linkType": "ContentType",
                                "id": "1t9IbcfdCk6m04uISSsaIK"
                            }
                        }
                    },
                    "fields": {
                        "name": {
                            "en-US": "San Francisco"
                        },
                        "center": {
                            "en-US": {
                                "lon": -122.41941550000001,
                                "lat": 37.7749295
                            }
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "ge1xHyH3QOWucKWCCAgIG",
                        "type": "Entry",
                        "createdAt": "2014-02-21T13:43:23.210Z",
                        "updatedAt": "2014-02-21T13:43:23.210Z",
                        "revision": 1,
                        "contentType": {
                            "sys": {
                                "type": "Link",
                                "linkType": "ContentType",
                                "id": "1t9IbcfdCk6m04uISSsaIK"
                            }
                        }
                    },
                    "fields": {
                        "name": {
                            "en-US": "Paris"
                        },
                        "center": {
                            "en-US": {
                                "lon": 2.3522219000000177,
                                "lat": 48.856614
                            }
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "4MU1s3potiUEM2G4okYOqw",
                        "type": "Entry",
                        "createdAt": "2014-02-21T13:42:45.926Z",
                        "updatedAt": "2014-02-21T13:42:45.926Z",
                        "revision": 1,
                        "contentType": {
                            "sys": {
                                "type": "Link",
                                "linkType": "ContentType",
                                "id": "1t9IbcfdCk6m04uISSsaIK"
                            }
                        }
                    },
                    "fields": {
                        "name": {
                            "en-US": "Berlin"
                        },
                        "center": {
                            "en-US": {
                                "lon": 13.404953999999975,
                                "lat": 52.52000659999999
                            }
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "1x0xpXu4pSGS4OukSyWGUK",
                        "type": "Asset",
                        "createdAt": "2013-11-06T09:45:10.000Z",
                        "updatedAt": "2013-12-18T13:27:14.917Z",
                        "revision": 6
                    },
                    "fields": {
                        "title": {
                            "en-US": "Doge"
                        },
                        "description": {
                            "en-US": "nice picture"
                        },
                        "file": {
                            "en-US": {
                                "url": "//images.contentful.com/cfexampleapi/1x0xpXu4pSGS4OukSyWGUK/cc1239c6385428ef26f4180190532818/doge.jpg",
                                "details": {
                                    "size": 522943,
                                    "image": {
                                        "width": 5800,
                                        "height": 4350
                                    }
                                },
                                "fileName": "doge.jpg",
                                "contentType": "image/jpeg"
                            }
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "jake",
                        "type": "Entry",
                        "createdAt": "2013-06-27T22:46:22.096Z",
                        "updatedAt": "2013-12-18T13:10:26.212Z",
                        "revision": 5,
                        "contentType": {
                            "sys": {
                                "type": "Link",
                                "linkType": "ContentType",
                                "id": "dog"
                            }
                        }
                    },
                    "fields": {
                        "name": {
                            "en-US": "Jake"
                        },
                        "description": {
                            "en-US": "Bacon pancakes, makin' bacon pancakes!"
                        },
                        "image": {
                            "en-US": {
                                "sys": {
                                    "type": "Link",
                                    "linkType": "Asset",
                                    "id": "jake"
                                }
                            }
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "happycat",
                        "type": "Entry",
                        "createdAt": "2013-06-27T22:46:20.171Z",
                        "updatedAt": "2013-11-18T15:58:02.018Z",
                        "revision": 8,
                        "contentType": {
                            "sys": {
                                "type": "Link",
                                "linkType": "ContentType",
                                "id": "cat"
                            }
                        }
                    },
                    "fields": {
                        "name": {
                            "en-US": "Happy Cat",
                            "tlh": "Quch vIghro'"
                        },
                        "likes": {
                            "en-US": [
                                "cheezburger"
                            ]
                        },
                        "color": {
                            "en-US": "gray"
                        },
                        "bestFriend": {
                            "en-US": {
                                "sys": {
                                    "type": "Link",
                                    "linkType": "Entry",
                                    "id": "nyancat"
                                }
                            }
                        },
                        "birthday": {
                            "en-US": "2003-10-28T23:00:00+00:00"
                        },
                        "lives": {
                            "en-US": 1
                        },
                        "image": {
                            "en-US": {
                                "sys": {
                                    "type": "Link",
                                    "linkType": "Asset",
                                    "id": "happycat"
                                }
                            }
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "6KntaYXaHSyIw8M6eo26OK",
                        "type": "Entry",
                        "createdAt": "2013-11-06T09:45:27.475Z",
                        "updatedAt": "2013-11-18T09:13:37.808Z",
                        "revision": 2,
                        "contentType": {
                            "sys": {
                                "type": "Link",
                                "linkType": "ContentType",
                                "id": "dog"
                            }
                        }
                    },
                    "fields": {
                        "name": {
                            "en-US": "Doge"
                        },
                        "description": {
                            "en-US": "such json\nwow"
                        },
                        "image": {
                            "en-US": {
                                "sys": {
                                    "type": "Link",
                                    "linkType": "Asset",
                                    "id": "1x0xpXu4pSGS4OukSyWGUK"
                                }
                            }
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "finn",
                        "type": "Entry",
                        "createdAt": "2013-06-27T22:46:21.450Z",
                        "updatedAt": "2013-09-09T16:15:01.297Z",
                        "revision": 6,
                        "contentType": {
                            "sys": {
                                "type": "Link",
                                "linkType": "ContentType",
                                "id": "human"
                            }
                        }
                    },
                    "fields": {
                        "name": {
                            "en-US": "Finn"
                        },
                        "description": {
                            "en-US": "Fearless adventurer! Defender of pancakes."
                        },
                        "likes": {
                            "en-US": [
                                "adventure"
                            ]
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "nyancat",
                        "type": "Entry",
                        "createdAt": "2013-06-27T22:46:19.513Z",
                        "updatedAt": "2013-09-04T09:19:39.027Z",
                        "revision": 5,
                        "contentType": {
                            "sys": {
                                "type": "Link",
                                "linkType": "ContentType",
                                "id": "cat"
                            }
                        }
                    },
                    "fields": {
                        "name": {
                            "en-US": "Nyan Cat",
                            "tlh": "Nyan vIghro'"
                        },
                        "likes": {
                            "en-US": [
                                "rainbows",
                                "fish"
                            ]
                        },
                        "color": {
                            "en-US": "rainbow"
                        },
                        "bestFriend": {
                            "en-US": {
                                "sys": {
                                    "type": "Link",
                                    "linkType": "Entry",
                                    "id": "happycat"
                                }
                            }
                        },
                        "birthday": {
                            "en-US": "2011-04-04T22:00:00+00:00"
                        },
                        "lives": {
                            "en-US": 1337
                        },
                        "image": {
                            "en-US": {
                                "sys": {
                                    "type": "Link",
                                    "linkType": "Asset",
                                    "id": "nyancat"
                                }
                            }
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "jake",
                        "type": "Asset",
                        "createdAt": "2013-09-02T14:56:34.260Z",
                        "updatedAt": "2013-09-02T15:22:39.466Z",
                        "revision": 2
                    },
                    "fields": {
                        "title": {
                            "en-US": "Jake"
                        },
                        "file": {
                            "en-US": {
                                "url": "//images.contentful.com/cfexampleapi/4hlteQAXS8iS0YCMU6QMWg/2a4d826144f014109364ccf5c891d2dd/jake.png",
                                "details": {
                                    "size": 20480,
                                    "image": {
                                        "width": 100,
                                        "height": 161
                                    }
                                },
                                "fileName": "jake.png",
                                "contentType": "image/png"
                            }
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "happycat",
                        "type": "Asset",
                        "createdAt": "2013-09-02T14:56:34.267Z",
                        "updatedAt": "2013-09-02T15:11:24.361Z",
                        "revision": 2
                    },
                    "fields": {
                        "title": {
                            "en-US": "Happy Cat"
                        },
                        "file": {
                            "en-US": {
                                "url": "//images.contentful.com/cfexampleapi/3MZPnjZTIskAIIkuuosCss/382a48dfa2cb16c47aa2c72f7b23bf09/happycatw.jpg",
                                "details": {
                                    "size": 59939,
                                    "image": {
                                        "width": 273,
                                        "height": 397
                                    }
                                },
                                "fileName": "happycatw.jpg",
                                "contentType": "image/jpeg"
                            }
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "nyancat",
                        "type": "Asset",
                        "createdAt": "2013-09-02T14:56:34.240Z",
                        "updatedAt": "2013-09-02T14:56:34.240Z",
                        "revision": 1
                    },
                    "fields": {
                        "title": {
                            "en-US": "Nyan Cat"
                        },
                        "file": {
                            "en-US": {
                                "url": "//images.contentful.com/cfexampleapi/4gp6taAwW4CmSgumq2ekUm/9da0cd1936871b8d72343e895a00d611/Nyan_cat_250px_frame.png",
                                "details": {
                                    "size": 12273,
                                    "image": {
                                        "width": 250,
                                        "height": 250
                                    }
                                },
                                "fileName": "Nyan_cat_250px_frame.png",
                                "contentType": "image/png"
                            }
                        }
                    }
                },
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "garfield",
                        "type": "Entry",
                        "createdAt": "2013-06-27T22:46:20.821Z",
                        "updatedAt": "2013-08-27T10:09:07.929Z",
                        "revision": 2,
                        "contentType": {
                            "sys": {
                                "type": "Link",
                                "linkType": "ContentType",
                                "id": "cat"
                            }
                        }
                    },
                    "fields": {
                        "name": {
                            "en-US": "Garfield",
                            "tlh": "Garfield"
                        },
                        "likes": {
                            "en-US": [
                                "lasagna"
                            ]
                        },
                        "color": {
                            "en-US": "orange"
                        },
                        "birthday": {
                            "en-US": "1979-06-18T23:00:00+00:00"
                        },
                        "lifes": {
                            "en-US": 1
                        },
                        "lives": {
                            "en-US": 9
                        }
                    }
                }
            ],
            "nextPageUrl": (
                "https://cdn.contentful.com/spaces/cfexampleapi/environments/master/sync?"
                "sync_token=w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-"
                "wo7CnDjChMKKGsK1wrzCrBzCqMOpZAwOOcOvCcOAwqHDv0XCiMKaOcOxZA8BJUzDr8K-wo1lNx7DnHE"
            )
        })

        self.assertEqual(
            sync_page.next_page_url,
            "https://cdn.contentful.com/spaces/cfexampleapi/environments/master/sync?"
            "sync_token=w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-"
            "wo7CnDjChMKKGsK1wrzCrBzCqMOpZAwOOcOvCcOAwqHDv0XCiMKaOcOxZA8BJUzDr8K-wo1lNx7DnHE"
        )
        self.assertEqual(
            sync_page.next_sync_token,
            "w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-"
            "wo7CnDjChMKKGsK1wrzCrBzCqMOpZAwOOcOvCcOAwqHDv0XCiMKaOcOxZA8BJUzDr8K-wo1lNx7DnHE"
        )
        self.assertEqual(len(sync_page.items), 14)
        self.assertEqual(str(sync_page.items[-1]), "<Entry[cat] id='garfield'>")
        self.assertEqual(
            str(sync_page.items[-2]),
            "<Asset id='nyancat' url='//images.contentful.com/cfexampleapi/4gp6taAwW4CmSgumq2ekUm/9da0cd1936871b8d72343e895a00d611/Nyan_cat_250px_frame.png'>"
        )

    def test__when_sync_has_multiple_pages__extracts_sync_token_from_next_page_url(self):
        sync_page = SyncPage({
            "sys": {
                "type": "Array"
            },
            "items": [
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "4rPdazIwWkuuKEAQgemSmO",
                        "type": "DeletedEntry",
                        "createdAt": "2014-08-11T08:30:42.546Z",
                        "updatedAt": "2014-08-11T08:30:42.546Z",
                        "deletedAt": "2014-08-11T08:30:42.546Z",
                        "revision": 1
                    }
                }
            ],
            "nextPageUrl": (
                "https://cdn.contentful.com/spaces/cfexampleapi/environments/master/sync?"
                "sync_token=w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-"
                "wo7CnDjChMKKGsK1w5zCrA3CnU7CgEvDtsK6w7B2wrRZwrwPIgDCjVo8PMOoUcK2wqTCl8O1wpY8wpjCkGM"
            )
        })

        self.assertEqual(
            sync_page.next_page_url,
            "https://cdn.contentful.com/spaces/cfexampleapi/environments/master/sync?"
            "sync_token=w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-"
            "wo7CnDjChMKKGsK1w5zCrA3CnU7CgEvDtsK6w7B2wrRZwrwPIgDCjVo8PMOoUcK2wqTCl8O1wpY8wpjCkGM"
        )

    def test__when_SyncPage_has_property_next_page_url__get_sync_token_extracts_token_from_it(self):
        sync_page = SyncPage({
            "sys": {
                "type": "Array"
            },
            "items": [
                {
                    "sys": {
                        "space": {
                            "sys": {
                                "type": "Link",
                                "linkType": "Space",
                                "id": "cfexampleapi"
                            }
                        },
                        "id": "4rPdazIwWkuuKEAQgemSmO",
                        "type": "DeletedEntry",
                        "createdAt": "2014-08-11T08:30:42.546Z",
                        "updatedAt": "2014-08-11T08:30:42.546Z",
                        "deletedAt": "2014-08-11T08:30:42.546Z",
                        "revision": 1
                    }
                }
            ],
            "nextPageUrl": (
                "https://cdn.contentful.com/spaces/cfexampleapi/environments/master/sync?"
                "sync_token=w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-"
                "wo7CnDjChMKKGsK1w5zCrA3CnU7CgEvDtsK6w7B2wrRZwrwPIgDCjVo8PMOoUcK2wqTCl8O1wpY8wpjCkGM"
            )
        })

        self.assertEqual(
            sync_page._get_sync_token(),
            "w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-"
            "wo7CnDjChMKKGsK1w5zCrA3CnU7CgEvDtsK6w7B2wrRZwrwPIgDCjVo8PMOoUcK2wqTCl8O1wpY8wpjCkGM"
        )



from unittest import TestCase
from contentful.resource_builder import ResourceBuilder
from contentful.sync_page import SyncPage


class ResourceBuilderTest(TestCase):
    def test__when_build_recieves_a_sync_page__returns_a_SyncPage(self):
        sync_api_responses = [
            {
                "sys": {
                    "type": "Array"
                },
                "items": [],
                "nextSyncUrl": (
                    "https://cdn.contentful.com/spaces/cfexampleapi/environments/master/sync?"
                    "sync_token=w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-"
                    "wo7CnDjChMKKGsK1w5zCrA3CnU7CgEvDtsK6w7B2wrRZwrwPIgDCjVo8PMOoUcK2wqTCl8O1wpY8wpjCkGM"
                )
            },
            {
                "sys": {
                    "type": "Array"
                },
                "items": [],
                "nextPageUrl": (
                    "https://cdn.contentful.com/spaces/cfexampleapi/environments/master/sync?"
                    "sync_token=w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-"
                    "wo7CnDjChMKKGsK1w5zCrA3CnU7CgEvDtsK6w7B2wrRZwrwPIgDCjVo8PMOoUcK2wqTCl8O1wpY8wpjCkGM"
                )
            }
        ]

        for response in sync_api_responses:
            resource_builder = ResourceBuilder(
                'en-GB',
                True,
                response
            )

            result = resource_builder.build()
            self.assertTrue(isinstance(result, SyncPage))



from unittest import TestCase
from contentful.locale import Locale


class LocaleTest(TestCase):
    def test_locale(self):
        locale = Locale({
            'code': 'en-US',
            'name': 'English U.S.',
            'fallbackCode': None,
            'default': True
        })

        self.assertEqual(locale.code, 'en-US')
        self.assertEqual(locale.name, 'English U.S.')
        self.assertEqual(locale.fallback_code, None)
        self.assertEqual(locale.default, True)
        self.assertEqual(str(locale), "<Locale[English U.S.] code='en-US' default=True fallback_code=None optional=False>")



from unittest import TestCase
from contentful.content_type_field import ContentTypeField


class ContentTypeFieldTest(TestCase):
    def test_content_type_field(self):
        content_type_field = ContentTypeField({
            "id": "name",
            "name": "Name",
            "type": "Text",
            "localized": True,
            "required": True,
            "disabled": False,
            "omitted": False
        })

        self.assertEqual(content_type_field.id, 'name')
        self.assertEqual(content_type_field.name, 'Name')
        self.assertEqual(content_type_field.type, 'Text')
        self.assertEqual(content_type_field.localized, True)
        self.assertEqual(content_type_field.omitted, False)
        self.assertEqual(content_type_field.required, True)
        self.assertEqual(content_type_field.disabled, False)
        self.assertEqual(str(content_type_field), "<ContentTypeField[Name] id='name' type='Text'>")

    def test_content_type_array_field(self):
        content_type_field = ContentTypeField({
            "id": "likes",
            "name": "Likes",
            "type": "Array",
            "localized": False,
            "required": False,
            "disabled": False,
            "omitted": False,
            "items": {
                "type": "Symbol",
                "validations": []
            }
        })

        self.assertEqual(content_type_field.id, 'likes')
        self.assertEqual(content_type_field.name, 'Likes')
        self.assertEqual(content_type_field.type, 'Array')
        self.assertEqual(content_type_field.items, {'type': 'Symbol', 'validations': []})
        self.assertEqual(content_type_field.localized, False)
        self.assertEqual(content_type_field.omitted, False)
        self.assertEqual(content_type_field.required, False)
        self.assertEqual(content_type_field.disabled, False)
        self.assertEqual(str(content_type_field), "<ContentTypeField[Likes] id='likes' type='Array'>")

    def test_none_values(self):
        field_types = [
            'Symbol',
            'Text',
            'Integer',
            'Number',
            'Date',
            'Boolean',
            'Location',
            'Link',
            'Array',
            'Object'
        ]
        for field_type in field_types:
            ct_field = ContentTypeField({
                "id": "likes",
                "name": "Likes",
                "type": field_type,
                "localized": False,
                "required": False,
                "disabled": False,
                "omitted": False,
                "items": {
                    "type": "Symbol",
                    "validations": []
                }
            })

            self.assertEqual(ct_field.coerce(None), None)



import vcr
import pickle
from datetime import datetime
from unittest import TestCase
from contentful.resource import Resource, FieldsResource, Link
from contentful.client import Client


class ResourceTest(TestCase):
    def test_resource(self):
        resource = Resource({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'bar'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1
            }
        })

        self.assertEqual(str(resource.space), "<Link[Space] id='foo'>")
        self.assertEqual(str(resource.content_type), "<Link[ContentType] id='bar'>")
        self.assertEqual(resource.created_at, datetime(2016, 6, 6))
        self.assertEqual(resource.updated_at, datetime(2016, 6, 6))
        self.assertEqual(resource.deleted_at, datetime(2016, 6, 6))
        self.assertEqual(resource.id, 'foobar')
        self.assertEqual(resource.version, 1)

        self.assertRaises(AttributeError, resource.__getattr__, 'foo')


class FieldsResourceTest(TestCase):
    def test_fields_resource(self):
        resource = FieldsResource({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'bar'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1,
                'locale': 'foo-locale'
            },
            'fields': {
                'foo': 'bar',
                'baz': 123,
                'qux': True
            }
        })

        self.assertEqual(resource.foo, 'bar')
        self.assertEqual(resource.baz, 123)
        self.assertEqual(resource.qux, True)
        self.assertEqual(resource.fields(), {'foo': 'bar', 'baz': 123, 'qux': True})
        self.assertEqual(resource.fields('foo-locale'), {'foo': 'bar', 'baz': 123, 'qux': True})
        self.assertEqual(resource.fields('bar-locale'), {})

        self.assertRaises(AttributeError, resource.__getattr__, 'foobar')

    def test_pickleable_resource(self):
        resource = FieldsResource({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'bar'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1,
                'locale': 'foo-locale'
            },
            'fields': {
                'foo': 'bar',
                'baz': 123,
                'qux': True
            }
        })

        serialized_resource = pickle.dumps(resource)
        deserialized_resource = pickle.loads(serialized_resource)

        self.assertEqual(resource.foo, deserialized_resource.foo)
        self.assertEqual(resource.baz, deserialized_resource.baz)
        self.assertEqual(resource.qux, deserialized_resource.qux)
        self.assertEqual(resource.fields(), deserialized_resource.fields())
        self.assertEqual(resource.fields('foo-locale'), deserialized_resource.fields('foo-locale'))
        self.assertEqual(resource.fields('bar-locale'), deserialized_resource.fields('bar-locale'))

        self.assertRaises(AttributeError, resource.__getattr__, 'foobar')

    def test_fields_resource_localized(self):
        resource = FieldsResource({
            'sys': {
                'space': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'Space',
                        'id': 'foo'
                    }
                },
                'contentType': {
                    'sys': {
                        'type': 'Link',
                        'linkType': 'ContentType',
                        'id': 'bar'
                    }
                },
                'type': 'Entry',
                'createdAt': '2016-06-06',
                'updatedAt': '2016-06-06',
                'deletedAt': '2016-06-06',
                'id': 'foobar',
                'version': 1,
                'locale': 'foo-locale'
            },
            'fields': {
                'foo': {'foo-locale': 'bar-foo', 'bar-locale': 'bar-bar'},
                'baz': {'foo-locale': 123, 'bar-locale': 456},
                'qux': {'foo-locale': True, 'bar-locale': False}
            }
        },
        localized=True)

        self.assertEqual(resource.foo, 'bar-foo')
        self.assertEqual(resource.baz, 123)
        self.assertEqual(resource.qux, True)
        self.assertEqual(resource.fields(), {'foo': 'bar-foo', 'baz': 123, 'qux': True})
        self.assertEqual(resource.fields('foo-locale'), {'foo': 'bar-foo', 'baz': 123, 'qux': True})
        self.assertEqual(resource.fields('bar-locale'), {'foo': 'bar-bar', 'baz': 456, 'qux': False})
        self.assertEqual(resource.fields('baz-locale'), {})

        self.assertRaises(AttributeError, resource.__getattr__, 'foobar')


class LinkTest(TestCase):
    def test_link(self):
        link = Link({
            'sys': {
                'type': 'Link',
                'linkType': 'Space',
                'id': 'foo'
            }
        })

        self.assertEqual(link.id, 'foo')
        self.assertEqual(link.type, 'Link')
        self.assertEqual(link.link_type, 'Space')
        self.assertEqual(str(link), "<Link[Space] id='foo'>")

    @vcr.use_cassette('fixtures/link/resolve_space.yaml', decode_compressed_response=True)
    def test_link_space_resource(self):
        link = Link({
            'sys': {
                'type': 'Link',
                'linkType': 'Space',
                'id': 'cfexampleapi'
            }
        })
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)

        space = link.resolve(client)

        self.assertEqual(str(space), "<Space[Contentful Example API] id='cfexampleapi'>")

    @vcr.use_cassette('fixtures/link/resolve_other.yaml', decode_compressed_response=True)
    def test_link_other_resource(self):
        link = Link({
            'sys': {
                'type': 'Link',
                'linkType': 'ContentType',
                'id': 'cat'
            }
        })
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)

        cat_ct = link.resolve(client)

        self.assertEqual(str(cat_ct), "<ContentType[Cat] id='cat'>")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import requests_mock
import vcr
import re
from unittest import TestCase

from requests_mock import ANY

from contentful.client import Client
from contentful.content_type_cache import ContentTypeCache
from contentful.errors import EntryNotFoundError
from contentful.errors import HTTPError
from contentful.utils import ConfigurationException
from contentful.entry import Entry


class ClientTest(TestCase):
    def setUp(self):
        ContentTypeCache.__CACHE__ = {}

    def test_client_repr(self):
        self.assertEqual(
            '<contentful.Client space_id="cfexampleapi" access_token="b4c0n73n7fu1" default_locale="en-US">',
            str(Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False))
        )

    def test_client_validations(self):
        with self.assertRaises(ConfigurationException):
            Client(None, 'foo')
        with self.assertRaises(ConfigurationException):
            Client('foo', None)
        with self.assertRaises(ConfigurationException):
            Client('foo', 'bar', api_url=None)
        with self.assertRaises(ConfigurationException):
            Client('foo', 'bar', default_locale=None)
        with self.assertRaises(ConfigurationException):
            Client('foo', 'bar', api_version=None)

    def test_uses_timeouts(self):
        c = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        with requests_mock.mock() as m:
            m.register_uri('GET', ANY, status_code=500)
            self.assertRaises(HTTPError, c.entries)
            self.assertEqual(m.call_count, 1)
            self.assertEqual(m.request_history[0].timeout, 1)

        c = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False, timeout_s=0.1231570235)
        with requests_mock.mock() as m:
            m.register_uri('GET', ANY, status_code=500)
            self.assertRaises(HTTPError, c.entries)
            self.assertEqual(m.call_count, 1)
            self.assertEqual(m.request_history[0].timeout, c.timeout_s)

    @vcr.use_cassette('fixtures/client/content_type_cache.yaml', decode_compressed_response=True)
    def test_client_creates_a_content_type_cache(self):
        Client('cfexampleapi', 'b4c0n73n7fu1')

        self.assertTrue(len(ContentTypeCache.__CACHE__) > 0)

    def test_client_can_avoid_caching_content_types(self):
        Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)

        self.assertFalse(len(ContentTypeCache.__CACHE__) > 0)

    @vcr.use_cassette('fixtures/client/space.yaml', decode_compressed_response=True)
    def test_client_get_space(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        space = client.space()

        self.assertEqual(str(space), "<Space[Contentful Example API] id='cfexampleapi'>")

    @vcr.use_cassette('fixtures/client/content_type.yaml', decode_compressed_response=True)
    def test_client_get_content_type(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        ct = client.content_type('cat')

        self.assertEqual(str(ct), "<ContentType[Cat] id='cat'>")

    @vcr.use_cassette('fixtures/client/content_types.yaml', decode_compressed_response=True)
    def test_client_get_content_types(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        cts = client.content_types()

        self.assertEqual(str(cts[0]), "<ContentType[City] id='1t9IbcfdCk6m04uISSsaIK'>")

    @vcr.use_cassette('fixtures/client/entry.yaml', decode_compressed_response=True)
    def test_client_entry(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        entry = client.entry('nyancat')

        self.assertEqual(str(entry), "<Entry[cat] id='nyancat'>")
        self.assertEqual(str(entry.best_friend), "<Entry[cat] id='happycat'>")

    @vcr.use_cassette('fixtures/client/entry_not_found.yaml', decode_compressed_response=True)
    def test_client_entry_not_found(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        self.assertRaises(EntryNotFoundError, client.entry, 'foobar')

    @vcr.use_cassette('fixtures/client/entries.yaml', decode_compressed_response=True)
    def test_client_entries(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        entries = client.entries()

        self.assertEqual(str(entries[0]), "<Entry[cat] id='happycat'>")

    @vcr.use_cassette('fixtures/client/entries_select.yaml', decode_compressed_response=True)
    def test_client_entries_select(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        entries = client.entries({'content_type': 'cat', 'sys.id': 'nyancat', 'select': ['fields.name']})

        self.assertEqual(str(entries[0]), "<Entry[cat] id='nyancat'>")
        self.assertEqual(entries[0].fields(), {'name': 'Nyan Cat'})

    @vcr.use_cassette('fixtures/client/entries_links_to_entry.yaml', decode_compressed_response=True)
    def test_client_entries_links_to_entry(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        entries = client.entries({'links_to_entry': 'nyancat'})
        self.assertEqual(len(entries), 1)
        self.assertEqual(str(entries[0]), "<Entry[cat] id='happycat'>")

    @vcr.use_cassette('fixtures/client/entry_incoming_references.yaml', decode_compressed_response=True)
    def test_entry_incoming_references(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        entry = client.entry('nyancat')
        entries = entry.incoming_references(client)
        self.assertEqual(len(entries), 1)
        self.assertEqual(str(entries[0]), "<Entry[cat] id='happycat'>")

    @vcr.use_cassette('fixtures/client/entry_incoming_references_with_query.yaml', decode_compressed_response=True)
    def test_entry_incoming_references_with_query(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        entry = client.entry('nyancat')
        entries = entry.incoming_references(client, {'content_type': 'cat', 'select': ['fields.name']})
        self.assertEqual(len(entries), 1)
        self.assertEqual(str(entries[0]), "<Entry[cat] id='happycat'>")
        self.assertEqual(entries[0].fields(), {'name': 'Happy Cat'})

    @vcr.use_cassette('fixtures/client/entries_links_to_asset.yaml', decode_compressed_response=True)
    def test_client_entries_links_to_asset(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        entries = client.entries({'links_to_asset': 'nyancat'})
        self.assertEqual(len(entries), 1)
        self.assertEqual(str(entries[0]), "<Entry[cat] id='nyancat'>")

    @vcr.use_cassette('fixtures/client/asset_incoming_references.yaml', decode_compressed_response=True)
    def test_asset_incoming_references(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        asset = client.asset('nyancat')
        entries = asset.incoming_references(client)
        self.assertEqual(len(entries), 1)
        self.assertEqual(str(entries[0]), "<Entry[cat] id='nyancat'>")

    @vcr.use_cassette('fixtures/client/asset.yaml', decode_compressed_response=True)
    def test_client_asset(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        asset = client.asset('nyancat')

        self.assertEqual(
            str(asset),
            "<Asset id='nyancat' url='//images.contentful.com/cfexampleapi/4gp6taAwW4CmSgumq2ekUm/9da0cd1936871b8d72343e895a00d611/Nyan_cat_250px_frame.png'>"
        )

    @vcr.use_cassette('fixtures/client/locales_on_environment.yaml', decode_compressed_response=True)
    def test_client_locales_on_environment(self):
        client = Client('facgnwwgj5fe', '<ACCESS_TOKEN>', environment='testing', content_type_cache=False)
        locales = client.locales()

        self.assertEqual(str(locales), "<Array size='3' total='3' limit='1000' skip='0'>")
        self.assertEqual(str(locales[0]), "<Locale[U.S. English] code='en-US' default=True fallback_code=None optional=False>")

    @vcr.use_cassette('fixtures/client/assets.yaml', decode_compressed_response=True)
    def test_client_assets(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        assets = client.assets()

        self.assertEqual(
            str(assets[0]),
            "<Asset id='1x0xpXu4pSGS4OukSyWGUK' url='//images.contentful.com/cfexampleapi/1x0xpXu4pSGS4OukSyWGUK/cc1239c6385428ef26f4180190532818/doge.jpg'>"
        )

    @vcr.use_cassette('fixtures/client/sync.yaml', decode_compressed_response=True)
    def test_client_sync(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        sync = client.sync({'initial': True})

        self.assertEqual(
            str(sync),
            "<SyncPage next_sync_token='{0}'>".format(
                'w5ZGw6JFwqZmVcKsE8Kow4grw45QdybCnV_Cg8OASMKpwo1UY8K8bsKFwqJrw7DDhcKnM2RDOVbDt1E-wo7CnDjChMKKGsK1wrzCrBzCqMOpZAwOOcOvCcOAwqHDv0XCiMKaOcOxZA8BJUzDr8K-wo1lNx7DnHE'
            )
        )
        self.assertEqual(str(sync.items[0]), "<Entry[1t9IbcfdCk6m04uISSsaIK] id='5ETMRzkl9KM4omyMwKAOki'>")

    @vcr.use_cassette('fixtures/client/sync_environments.yaml', decode_compressed_response=True)
    def test_client_sync_with_environments(self):
        client = Client('a22o2qgm356c', 'bfbc63cf745a037125dbcc64f716a9a0e9d091df1a79e84920b890f87a6e7ab9', environment='staging', content_type_cache=False)
        sync = client.sync({'initial': True})

        self.assertEqual(sync.items[0].environment.id, 'staging')

    @vcr.use_cassette('fixtures/client/array_endpoints.yaml', decode_compressed_response=True)
    def test_client_creates_wrapped_arrays(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)
        self.assertEqual(str(client.content_types()), "<Array size='4' total='4' limit='100' skip='0'>")
        self.assertEqual(str(client.entries()), "<Array size='10' total='10' limit='100' skip='0'>")
        self.assertEqual(str(client.assets()), "<Array size='4' total='4' limit='100' skip='0'>")

    # X-Contentful-User-Agent Headers

    def test_client_default_contentful_user_agent_headers(self):
        client = Client('cfexampleapi', 'b4c0n73n7fu1', content_type_cache=False)

        from contentful import __version__
        import platform
        expected = [
            'sdk contentful.py/{0};'.format(__version__),
            'platform python/{0};'.format(platform.python_version())
        ]
        header = client._contentful_user_agent()
        for e in expected:
            self.assertTrue(e in header)

        self.assertTrue(re.search(r'os (Windows|macOS|Linux)(/.*)?;', header))

        self.assertTrue('integration' not in header)
        self.assertTrue('app' not in header)

    def test_client_with_integration_name_only_headers(self):
        client = Client(
            'cfexampleapi',
            'b4c0n73n7fu1',
            content_type_cache=False,
            integration_name='foobar')

        header = client._contentful_user_agent()
        self.assertTrue('integration foobar;' in header)
        self.assertFalse('integration foobar/;' in header)

    def test_client_with_integration_headers(self):
        client = Client(
            'cfexampleapi',
            'b4c0n73n7fu1',
            content_type_cache=False,
            integration_name='foobar',
            integration_version='0.1.0')

        header = client._contentful_user_agent()
        self.assertTrue('integration foobar/0.1.0;' in header)

    def test_client_with_application_name_only_headers(self):
        client = Client(
            'cfexampleapi',
            'b4c0n73n7fu1',
            content_type_cache=False,
            application_name='foobar')

        header = client._contentful_user_agent()
        self.assertTrue('app foobar;' in header)
        self.assertFalse('app foobar/;' in header)

    def test_client_with_application_headers(self):
        client = Client(
            'cfexampleapi',
            'b4c0n73n7fu1',
            content_type_cache=False,
            application_name='foobar',
            application_version='0.1.0')

        header = client._contentful_user_agent()
        self.assertTrue('app foobar/0.1.0;' in header)

    def test_client_with_integration_version_only_does_not_include_integration_in_header(self):
        client = Client(
            'cfexampleapi',
            'b4c0n73n7fu1',
            content_type_cache=False,
            integration_version='0.1.0')

        header = client._contentful_user_agent()
        self.assertFalse('integration /0.1.0' in header)

    def test_client_with_application_version_only_does_not_include_integration_in_header(self):
        client = Client(
            'cfexampleapi',
            'b4c0n73n7fu1',
            content_type_cache=False,
            application_version='0.1.0')

        header = client._contentful_user_agent()
        self.assertFalse('app /0.1.0;' in header)

    def test_client_with_all_headers(self):
        client = Client(
            'cfexampleapi',
            'b4c0n73n7fu1',
            content_type_cache=False,
            application_name='foobar_app',
            application_version='1.1.0',
            integration_name='foobar integ',
            integration_version='0.1.0')

        from contentful import __version__
        import platform
        expected = [
            'sdk contentful.py/{0};'.format(__version__),
            'platform python/{0};'.format(platform.python_version()),
            'app foobar_app/1.1.0;',
            'integration foobar integ/0.1.0;'
        ]
        header = client._contentful_user_agent()
        for e in expected:
            self.assertTrue(e in header)

        self.assertTrue(re.search(r'os (Windows|macOS|Linux)(/.*)?;', header))

    def test_client_headers(self):
        client = Client(
            'cfexampleapi',
            'b4c0n73n7fu1',
            content_type_cache=False,
            application_name='foobar_app',
            application_version='1.1.0',
            integration_name='foobar integ',
            integration_version='0.1.0')

        from contentful import __version__
        import platform
        expected = [
            'sdk contentful.py/{0};'.format(__version__),
            'platform python/{0};'.format(platform.python_version()),
            'app foobar_app/1.1.0;',
            'integration foobar integ/0.1.0;'
        ]
        header = client._request_headers()['X-Contentful-User-Agent']
        for e in expected:
            self.assertTrue(e in header)

        self.assertTrue(re.search(r'os (Windows|macOS|Linux)(/.*)?;', header))

    # Integration Tests

    @vcr.use_cassette('fixtures/integration/issue-4.yaml', decode_compressed_response=True)
    def test_entries_dont_fail_with_unicode_characters(self):
        client = Client('wltm0euukdog', 'bbe871957bb60f988af6cbeeccbb178c36cae09e36e8098357e27b51dd38d88e', content_type_cache=True)
        entries = client.entries()
        self.assertEqual(entries[0].name, '😅')

    @vcr.use_cassette('fixtures/integration/json-arrays.yaml', decode_compressed_response=True)
    def test_entries_dont_fail_with_arrays_as_json_root(self):
        client = Client('4int1zgmkwcf', 'd2ac2076019bd4a8357811cbdd5563bb7186d90d77e53c265a1bafd9f83439e8', content_type_cache=True)
        entries = client.entries()
        self.assertEqual(entries[0].json, [{'foo': 'bar'}, {'baz': 'qux'}])

    @vcr.use_cassette('fixtures/integration/issue-11.yaml', decode_compressed_response=True)
    def test_entries_with_none_values_on_all_fields(self):
        client = Client('rtx5c7z0zbas', 'a6c8dc438d470c51d1094dad146a1f20fcdba41e21f4e263af6c3f70d8583634', content_type_cache=True)
        entry = client.entries()[0]
        self.assertEqual(entry.symbol, None)
        self.assertEqual(entry.text, None)
        self.assertEqual(entry.integer, None)
        self.assertEqual(entry.number, None)
        self.assertEqual(entry.date, None)
        self.assertEqual(entry.location, None)
        self.assertEqual(entry.asset, None)
        self.assertEqual(entry.bool, None)
        self.assertEqual(entry.json, None)
        self.assertEqual(entry.link, None)

    @vcr.use_cassette('fixtures/integration/circular-references.yaml', decode_compressed_response=True)
    def test_circular_references_default_depth(self):
        client = Client('rk19fq93y3vw', '821aa502a7ce820e46adb30fa6942889619aac4342a7021cfe15197c52a593cc', content_type_cache=True)
        a = client.entry('6kdfS7uMs8owuEIoSaOcQk')
        self.assertEqual(str(a), "<Entry[a] id='6kdfS7uMs8owuEIoSaOcQk'>")
        self.assertEqual(str(a.b), "<Entry[b] id='7oADpDPuneEAsWUiO2CmEo'>")
        self.assertEqual(str(a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a), "<Entry[a] id='6kdfS7uMs8owuEIoSaOcQk'>")
        self.assertEqual(str(a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b), "<Link[Entry] id='7oADpDPuneEAsWUiO2CmEo'>")

    @vcr.use_cassette('fixtures/integration/circular-references.yaml', decode_compressed_response=True)
    def test_circular_references_set_depth(self):
        client = Client(
            'rk19fq93y3vw',
            '821aa502a7ce820e46adb30fa6942889619aac4342a7021cfe15197c52a593cc',
            content_type_cache=True,
            max_include_resolution_depth=1
        )
        a = client.entry('6kdfS7uMs8owuEIoSaOcQk')
        self.assertEqual(str(a), "<Entry[a] id='6kdfS7uMs8owuEIoSaOcQk'>")
        self.assertEqual(str(a.b), "<Entry[b] id='7oADpDPuneEAsWUiO2CmEo'>")
        self.assertEqual(str(a.b.a), "<Link[Entry] id='6kdfS7uMs8owuEIoSaOcQk'>")

    @vcr.use_cassette('fixtures/integration/circular-references.yaml', decode_compressed_response=True)
    def test_circular_references_with_reusable_entries(self):
        client = Client('rk19fq93y3vw', '821aa502a7ce820e46adb30fa6942889619aac4342a7021cfe15197c52a593cc', content_type_cache=True, reuse_entries=True)
        a = client.entry('6kdfS7uMs8owuEIoSaOcQk')
        self.assertEqual(str(a), "<Entry[a] id='6kdfS7uMs8owuEIoSaOcQk'>")
        self.assertEqual(str(a.b), "<Entry[b] id='7oADpDPuneEAsWUiO2CmEo'>")
        self.assertEqual(str(a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a), "<Entry[a] id='6kdfS7uMs8owuEIoSaOcQk'>")
        self.assertEqual(str(a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b), "<Entry[b] id='7oADpDPuneEAsWUiO2CmEo'>")
        self.assertEqual(a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b._depth, 1)
        self.assertEqual(a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a._depth, 0)
        self.assertEqual(str(a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a.b.a), "<Entry[a] id='6kdfS7uMs8owuEIoSaOcQk'>")

    @vcr.use_cassette('fixtures/integration/errors-filtered.yaml', decode_compressed_response=True)
    def test_unresolvable_entries_dont_get_included(self):
        client = Client(
            '011npgaszg5o',
            '42c9d93410a7319e9a735671fc1e415348f65e94a99fc768b70a7c649859d4fd'
        )

        entry = client.entry('1HR1QvURo4MoSqO0eqmUeO')
        self.assertEqual(len(entry.modules), 2)

    @vcr.use_cassette('fixtures/fields/rich_text.yaml', decode_compressed_response=True)
    def test_rich_text_field(self):
        client = Client(
            'jd7yc4wnatx3',
            '6256b8ef7d66805ca41f2728271daf27e8fa6055873b802a813941a0fe696248',
            gzip_encoded=False
        )

        entry = client.entry('4BupPSmi4M02m0U48AQCSM')

        expected_entry_occurrances = 2
        embedded_entry_index = 1
        for content in entry.body['content']:
            if content['nodeType'] == 'embedded-entry-block':
                self.assertTrue(isinstance(content['data']['target'], Entry))
                self.assertEqual(content['data']['target'].body, 'Embedded {0}'.format(embedded_entry_index))
                expected_entry_occurrances -= 1
                embedded_entry_index += 1
        self.assertEqual(expected_entry_occurrances, 0)

    @vcr.use_cassette('fixtures/fields/rich_text_lists_with_embeds.yaml', decode_compressed_response=True)
    def test_rich_text_field_with_embeds_in_lists(self):
        client = Client(
            'jd7yc4wnatx3',
            '6256b8ef7d66805ca41f2728271daf27e8fa6055873b802a813941a0fe696248',
            gzip_encoded=False
        )

        entry = client.entry('6NGLswCREsGA28kGouScyY')

        # Hyperlink data is conserved
        self.assertEqual(entry.body['content'][0], {
            'data': {},
            'content': [
                {'marks': [], 'value': 'A link to ', 'nodeType': 'text', 'nodeClass': 'text'},
                {
                    'data': {'uri': 'https://google.com'},
                    'content': [{'marks': [], 'value': 'google', 'nodeType': 'text', 'nodeClass': 'text'}],
                    'nodeType': 'hyperlink',
                    'nodeClass': 'inline'
                },
                {'marks': [], 'value': '', 'nodeType': 'text', 'nodeClass': 'text'}
            ],
            'nodeType': 'paragraph',
            'nodeClass': 'block'
        })

        # Unordered lists and ordered lists can contain embedded entries
        self.assertEqual(entry.body['content'][3]['nodeType'], 'unordered-list')
        self.assertEqual(str(entry.body['content'][3]['content'][2]['content'][0]['data']['target']), "<Entry[embedded] id='49rofLvvxCOiIMIi6mk8ai'>")

        self.assertEqual(entry.body['content'][4]['nodeType'], 'ordered-list')
        self.assertEqual(str(entry.body['content'][4]['content'][2]['content'][0]['data']['target']), "<Entry[embedded] id='5ZF9Q4K6iWSYIU2OUs0UaQ'>")

    @vcr.use_cassette('fixtures/integration/issue-41.yaml', decode_compressed_response=True)
    def test_rich_text_fields_should_not_get_hydrated_twice(self):
        client = Client(
            'fds721b88p6b',
            '45ba81cc69423fcd2e3f0a4779de29481bb5c11495bc7e14649a996cf984e98e',
            gzip_encoded=False
        )

        entry = client.entry('1tBAu0wP9qAQEg6qCqMics')

        # Not failing is already a success
        self.assertEqual(str(entry.children[0]), str(entry.children[1]))
        self.assertEqual(str(entry.children[0].body), str(entry.children[1].body))



from unittest import TestCase
from contentful.content_type import ContentType


class ContentTypeTest(TestCase):
    def test_content_type(self):
        content_type = ContentType({
            "sys": {
                "space": {
                    "sys": {
                        "type": "Link",
                        "linkType": "Space",
                        "id": "cfexampleapi"
                    }
                },
                "id": "cat",
                "type": "ContentType",
                "createdAt": "2013-06-27T22:46:12.852Z",
                "updatedAt": "2016-11-21T15:01:43.860Z",
                "revision": 6
            },
            "displayField": "name",
            "name": "Cat",
            "description": "Meow.",
            "fields": [
                {
                    "id": "name",
                    "name": "Name",
                    "type": "Text",
                    "localized": True,
                    "required": True,
                    "disabled": False,
                    "omitted": False
                },
                {
                    "id": "likes",
                    "name": "Likes",
                    "type": "Array",
                    "localized": False,
                    "required": False,
                    "disabled": False,
                    "omitted": False,
                    "items": {
                        "type": "Symbol",
                        "validations": []
                    }
                },
                {
                    "id": "color",
                    "name": "Color",
                    "type": "Symbol",
                    "localized": False,
                    "required": False,
                    "disabled": False,
                    "omitted": False
                },
                {
                    "id": "bestFriend",
                    "name": "Best Friend",
                    "type": "Link",
                    "localized": False,
                    "required": False,
                    "disabled": False,
                    "omitted": False,
                    "linkType": "Entry"
                },
                {
                    "id": "birthday",
                    "name": "Birthday",
                    "type": "Date",
                    "localized": False,
                    "required": False,
                    "disabled": False,
                    "omitted": False
                },
                {
                    "id": "lifes",
                    "name": "Lifes left",
                    "type": "Integer",
                    "localized": False,
                    "required": False,
                    "disabled": True,
                    "omitted": False
                },
                {
                    "id": "lives",
                    "name": "Lives left",
                    "type": "Integer",
                    "localized": False,
                    "required": False,
                    "disabled": False,
                    "omitted": False
                },
                {
                    "id": "image",
                    "name": "Image",
                    "type": "Link",
                    "localized": False,
                    "required": False,
                    "disabled": False,
                    "omitted": False,
                    "linkType": "Asset"
                }
            ]
        })

        self.assertEqual(content_type.name, 'Cat')
        self.assertEqual(content_type.description, 'Meow.')
        self.assertEqual(content_type.display_field, 'name')
        self.assertEqual(str(content_type.fields[0]), "<ContentTypeField[Name] id='name' type='Text'>")
        self.assertEqual(str(content_type), "<ContentType[Cat] id='cat'>")

        self.assertEqual(str(content_type.field_for('name')), "<ContentTypeField[Name] id='name' type='Text'>")



from unittest import TestCase
from contentful import utils


class UtilsTest(TestCase):
    def test_snake_case(self):
        self.assertEqual(utils.snake_case('foo'), 'foo')
        self.assertEqual(utils.snake_case('foo_bar'), 'foo_bar')
        self.assertEqual(utils.snake_case('Foo'), 'foo')
        self.assertEqual(utils.snake_case('FooBar'), 'foo_bar')
        self.assertEqual(utils.snake_case('fooBar'), 'foo_bar')

    def test_is_link(self):
        self.assertFalse(utils.is_link('foo'))
        self.assertFalse(utils.is_link([1, 2, 3]))
        self.assertFalse(utils.is_link({'foo': 'bar'}))
        self.assertFalse(utils.is_link({'sys': {'type': 'Space'}, 'id': 'foo'}))

        self.assertTrue(utils.is_link({'sys': {'type': 'Link'}, 'id': 'foo'}))

    def test_is_link_array(self):
        self.assertFalse(utils.is_link_array('foo'))
        self.assertFalse(utils.is_link_array([1, 2, 3]))
        self.assertFalse(utils.is_link_array({'foo': 'bar'}))
        self.assertFalse(utils.is_link_array({'sys': {'type': 'Space'}, 'id': 'foo'}))
        self.assertFalse(utils.is_link_array({'sys': {'type': 'Link'}, 'id': 'foo'}))
        self.assertFalse(utils.is_link_array([{'sys': {'type': 'Space'}, 'id': 'foo'}]))

        self.assertTrue(utils.is_link_array([{'sys': {'type': 'Link'}, 'id': 'foo'}]))

    def test_resource_for_link(self):
        includes = [
            {'sys': {'id': 'foo', 'type': 'Entry'}},
            {'sys': {'id': 'bar', 'type': 'Entry'}},
            {'sys': {'id': 'foo', 'type': 'Asset'}}
        ]

        foo_entry = utils.resource_for_link({'sys': {'type': 'Link', 'id': 'foo', 'linkType': 'Entry'}}, includes)
        self.assertEqual(foo_entry, includes[0])

        bar_entry = utils.resource_for_link({'sys': {'type': 'Link', 'id': 'bar', 'linkType': 'Entry'}}, includes)
        self.assertEqual(bar_entry, includes[1])

        foo_asset = utils.resource_for_link({'sys': {'type': 'Link', 'id': 'foo', 'linkType': 'Asset'}}, includes)
        self.assertEqual(foo_asset, includes[2])

    def test_resource_for_link_with_cached_resource(self):
        includes = []
        cached_resources = {"Entry:foo:*": "foobar"}

        foo_entry = utils.resource_for_link({'sys': {'type': 'Link', 'id': 'foo', 'linkType': 'Entry'}}, includes, resources=cached_resources, locale='*')
        self.assertEqual(foo_entry, cached_resources["Entry:foo:*"])

        bar_entry = utils.resource_for_link({'sys': {'type': 'Link', 'id': 'bar', 'linkType': 'Entry'}}, includes, resources=cached_resources, locale='*')
        self.assertIsNone(bar_entry)

    def _test_retry_request(self):
        """Tested as part of Errors"""
        pass

    def test_unresolvable_empty_item_returns_true(self):
        self.assertTrue(utils.unresolvable(None, []))
